"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var is_1 = __importDefault(require("@sindresorhus/is"));
var constants_1 = require("../types/constants");
function CallInjectedView(target, key) {
    var method = target[key];
    var constructor = Object.getPrototypeOf(target).constructor;
    var container = Reflect.getMetadata(constants_1.MODULE, constructor);
    var inject = Reflect.getMetadata(constants_1.VIEW, constructor, key) || [];
    var params = Reflect.getMetadata(constants_1.PARAMETER, constructor, key);
    if (params) {
        params.forEach(function (_a) {
            var index = _a.index, identifier = _a.identifier;
            inject.splice(index, 1, identifier);
        });
    }
    var dependencies = inject.map(function (i, index) {
        if (container.isBound(i)) {
            return container.get(i);
        }
        else {
            throw new Error("Can't resolve parameter [" + index + "] of " + constructor.name + ":" + key);
        }
    });
    if (dependencies.some(is_1.default.promise)) {
        return Promise.all(dependencies).then(function (args) {
            return method.apply(target, PerformParameterAction(args, params));
        });
    }
    else {
        return method.apply(target, PerformParameterAction(dependencies, params));
    }
}
exports.CallInjectedView = CallInjectedView;
function PerformParameterAction(args, actions) {
    if (!actions) {
        return args;
    }
    actions.forEach(function (transform) {
        var action = transform.action;
        args.splice(transform.index, 1, action(args[transform.index]));
    });
    return args;
}
exports.PerformParameterAction = PerformParameterAction;
